@@ -1,14 +1,10 @@
 import sys
 import warnings
-
-try:
-    import itertools.izip as zip
-except ImportError:
-    pass
-
 from itertools import product
 
 import numpy as np
+import pandas as pd
+from pandas.api.types import is_numeric_dtype
 
 from .. import util
 from ..dimension import dimension_name
@@ -53,7 +49,6 @@ class cuDFInterface(PandasInterface):
     @classmethod
     def init(cls, eltype, data, kdims, vdims):
         import cudf
-        import pandas as pd
 
         element_params = eltype.param.objects()
         kdim_param = element_params['kdims']
@@ -114,9 +109,9 @@ class cuDFInterface(PandasInterface):
             d = dimension_name(d)
             if len([c for c in columns if c == d]) > 1:
                 raise DataError('Dimensions may not reference duplicated DataFrame '
-                                'columns (found duplicate %r columns). If you want to plot '
+                                f'columns (found duplicate {d!r} columns). If you want to plot '
                                 'a column against itself simply declare two dimensions '
-                                'with the same name. '% d, cls)
+                                'with the same name.', cls)
         return data, {'kdims':kdims, 'vdims':vdims}, {}
 
 
@@ -127,7 +122,7 @@ class cuDFInterface(PandasInterface):
         if dimension.nodata is not None:
             column = cls.replace_value(column, dimension.nodata)
         if column.dtype.kind == 'O':
-            return np.NaN, np.NaN
+            return np.nan, np.nan
         else:
             return finite_range(column, column.min(), column.max())
 
@@ -199,10 +194,10 @@ class cuDFInterface(PandasInterface):
             if isinstance(sel, tuple):
                 sel = slice(*sel)
             arr = cls.values(dataset, dim, keep_index=True)
-            if util.isdatetime(arr) and util.pd:
+            if util.isdatetime(arr):
                 try:
                     sel = util.parse_datetime_selection(sel)
-                except:
+                except Exception:
                     pass
 
             new_masks = []
@@ -237,6 +232,22 @@ class cuDFInterface(PandasInterface):
                 mask &= new_mask
         return mask
 
+    @classmethod
+    def _select_mask_neighbor(cls, dataset, selection):
+        """Runs select mask and expand the True values to include its neighbors
+
+        Example
+
+        select_mask =          [False, False, True, True, False, False]
+        select_mask_neighbor = [False, True,  True, True, True,  False]
+
+        """
+        mask = cls.select_mask(dataset, selection).to_cupy()
+        extra = (mask[1:] ^ mask[:-1])
+        mask[1:] |= extra
+        mask[:-1] |= extra
+        return mask
+
     @classmethod
     def select(cls, dataset, selection_mask=None, **selection):
         df = dataset.data
@@ -274,22 +285,26 @@ class cuDFInterface(PandasInterface):
             agg = agg_map.get(agg, agg)
             grouped = reindexed.groupby(cols, sort=False)
             if not hasattr(grouped, agg):
-                raise ValueError('%s aggregation is not supported on cudf DataFrame.' % agg)
-            df = getattr(grouped, agg)().reset_index()
+                raise ValueError(f'{agg} aggregation is not supported on cudf DataFrame.')
+            numeric_cols = [
+                c for c, d in zip(reindexed.columns, reindexed.dtypes)
+                if is_numeric_dtype(d) and c not in cols
+            ]
+            df = getattr(grouped[numeric_cols], agg)().reset_index()
         else:
             agg_map = {'amin': 'min', 'amax': 'max', 'size': 'count'}
             agg = agg_map.get(agg, agg)
             if not hasattr(reindexed, agg):
-                raise ValueError('%s aggregation is not supported on cudf DataFrame.' % agg)
+                raise ValueError(f'{agg} aggregation is not supported on cudf DataFrame.')
             agg = getattr(reindexed, agg)()
             try:
-                data = dict(((col, [v]) for col, v in zip(agg.index.values_host, agg.to_numpy())))
+                data = {col: [v] for col, v in zip(agg.index.values_host, agg.to_numpy())}
             except Exception:
                 # Give FutureWarning: 'The to_array method will be removed in a future cuDF release.
                 # Consider using `to_numpy` instead.'
                 # Seen in cudf=21.12.01
-                data = dict(((col, [v]) for col, v in zip(agg.index.values_host, agg.to_array())))
-            df = util.pd.DataFrame(data, columns=list(agg.index.values_host))
+                data = {col: [v] for col, v in zip(agg.index.values_host, agg.to_array())}
+            df = pd.DataFrame(data, columns=list(agg.index.values_host))
 
         dropped = []
         for vd in vdims:
@@ -331,7 +346,9 @@ class cuDFInterface(PandasInterface):
 
 
     @classmethod
-    def sort(cls, dataset, by=[], reverse=False):
+    def sort(cls, dataset, by=None, reverse=False):
+        if by is None:
+            by = []
         cols = [dataset.get_dimension(d, strict=True).name for d in by]
         return dataset.data.sort_values(by=cols, ascending=not reverse)
 
