@@ -2,7 +2,6 @@ import warnings
 from collections.abc import Callable, Iterable
 from functools import partial
 
-import dask.dataframe as dd
 import datashader as ds
 import datashader.reductions as rd
 import datashader.transfer_functions as tf
@@ -45,6 +44,7 @@ from ..core.util import (
     datetime_types,
     dt_to_int,
     get_param_values,
+    lazy_isinstance,
 )
 from ..element import (
     RGB,
@@ -303,22 +303,23 @@ class aggregate(LineAggregationOperation):
         if len(paths) > 1:
             if glyph == 'line':
                 path = paths[0][:1]
-                if isinstance(path, dd.DataFrame):
+                if lazy_isinstance(path, "dask.dataframe:DataFrame"):
                     path = path.compute()
                 empty = path.copy()
                 empty.iloc[0, :] = (np.nan,) * empty.shape[1]
                 paths = [elem for p in paths for elem in (p, empty)][:-1]
-            if all(isinstance(path, dd.DataFrame) for path in paths):
+            if all(lazy_isinstance(path,"dask.dataframe:DataFrame") for path in paths):
+                import dask.dataframe as dd
                 df = dd.concat(paths)
             else:
-                paths = [p.compute() if isinstance(p, dd.DataFrame) else p for p in paths]
+                paths = [p.compute() if lazy_isinstance(p, "dask.dataframe:DataFrame") else p for p in paths]
                 df = pd.concat(paths)
         else:
             df = paths[0] if paths else pd.DataFrame([], columns=[x.name, y.name])
         if category and df[category].dtype.name != 'category':
             df[category] = df[category].astype('category')
 
-        is_custom = isinstance(df, dd.DataFrame) or cuDFInterface.applies(df)
+        is_custom = lazy_isinstance(df, "dask.dataframe:DataFrame") or cuDFInterface.applies(df)
         if any((not is_custom and len(df[d.name]) and isinstance(df[d.name].values[0], cftime_types)) or
                df[d.name].dtype.kind in ["M", "u"] for d in (x, y)):
             df = df.copy()
