@@ -1,44 +1,41 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 
+from colorcet import kbc, register_cmap
 from matplotlib import rc_params_from_file
-from matplotlib.colors import ListedColormap
+from matplotlib.colors import LinearSegmentedColormap, ListedColormap
+from packaging.version import Version
 from param import concrete_descendents
 
-from ...core import Layout, Collator, GridMatrix, config
-from ...core.options import Cycle, Palette, Options
+from ...core import Collator, GridMatrix, Layout, config
+from ...core.options import Cycle, Options, Palette
 from ...core.overlay import NdOverlay, Overlay
-from ...core.util import LooseVersion, pd
-from ...element import * # noqa (API import)
+from ...element import *
 from ..plot import PlotSelector
-from .annotation import * # noqa (API import)
-from .chart import * # noqa (API import)
-from .chart3d import * # noqa (API import)
+from ..util import fire_colors
+from .annotation import *
+from .chart import *
+from .chart3d import *
 from .element import ElementPlot
-from .geometry import * # noqa (API import)
-from .graphs import * # noqa (API import)
-from .heatmap import * # noqa (API import)
-from .hex_tiles import * # noqa (API import)
-from .path import * # noqa (API import)
-from .plot import * # noqa (API import)
-from .raster import * # noqa (API import)
-from .sankey import * # noqa (API import)
-from .stats import * # noqa (API import)
-from .tabular import * # noqa (API import)
-
+from .geometry import *
+from .graphs import *
+from .heatmap import *
+from .hex_tiles import *
+from .path import *
+from .plot import *
+from .raster import *
 from .renderer import MPLRenderer
+from .sankey import *
+from .stats import *
+from .tabular import *
 
+mpl_ge_150 = Version(mpl.__version__) >= Version('1.5.0')
 
-mpl_ge_150 = LooseVersion(mpl.__version__) >= '1.5.0'
-
-if pd:
-    try:
-        from pandas.plotting import register_matplotlib_converters
-        register_matplotlib_converters()
-    except ImportError:
-        from pandas.tseries import converter
-        converter.register()
+try:
+    from pandas.plotting import register_matplotlib_converters
+    register_matplotlib_converters()
+except ImportError:
+    from pandas.tseries import converter
+    converter.register()
 
 
 def set_style(key):
@@ -137,7 +134,6 @@ Store.register({Curve: CurvePlot,
                 # Chart 3D
                 Surface: SurfacePlot,
                 TriSurface: TriSurfacePlot,
-                Trisurface: TriSurfacePlot, # Alias, remove in 2.0
                 Scatter3D: Scatter3DPlot,
                 Path3D: Path3DPlot,
 
@@ -154,6 +150,7 @@ Store.register({Curve: CurvePlot,
                                        False: HeatMapPlot},
                                       True),
                 Image: RasterPlot,
+                ImageStack: RGBPlot,
                 RGB: RGBPlot,
                 HSV: RGBPlot,
 
@@ -166,6 +163,10 @@ Store.register({Curve: CurvePlot,
                 Sankey: SankeyPlot,
 
                 # Annotation plots
+                VLines: VLinesAnnotationPlot,
+                HLines: HLinesAnnotationPlot,
+                HSpans: HSpansAnnotationPlot,
+                VSpans: VSpansAnnotationPlot,
                 VLine: VLinePlot,
                 HLine: HLinePlot,
                 VSpan: VSpanPlot,
@@ -211,10 +212,21 @@ for framedcls in [VectorFieldPlot, ContourPlot, PathPlot, RasterPlot,
                   QuadMeshPlot, HeatMapPlot, PolygonPlot]:
     framedcls.show_frame = True
 
+fire_cmap   = LinearSegmentedColormap.from_list("fire",   fire_colors, N=len(fire_colors))
+fire_r_cmap = LinearSegmentedColormap.from_list("fire_r", list(reversed(fire_colors)),
+                                                N=len(fire_colors))
+register_cmap("fire", cmap=fire_cmap)
+register_cmap("fire_r", cmap=fire_r_cmap)
+
+register_cmap('kbc_r',
+              cmap=LinearSegmentedColormap.from_list('kbc_r',
+                                                     list(reversed(kbc)), N=len(kbc)))
+
 options = Store.options(backend='matplotlib')
-dflt_cmap = 'fire'
+dflt_cmap = config.default_cmap
+
 # Default option definitions
-# Note: *No*short aliases here! e.g use 'facecolor' instead of 'fc'
+# Note: *No*short aliases here! e.g. use 'facecolor' instead of 'fc'
 
 # Charts
 options.Curve = Options('style', color=Cycle(), linewidth=2)
@@ -230,7 +242,7 @@ options.Scatter3D = Options('plot', fig_size=150)
 options.Path3D = Options('plot', fig_size=150)
 options.Surface = Options('plot', fig_size=150)
 options.Surface = Options('style', cmap='fire')
-options.Spikes = Options('style', color='black', cmap='fire')
+options.Spikes = Options('style', color='black', cmap=dflt_cmap)
 options.Area = Options('style', facecolor=Cycle(), edgecolor='black')
 options.BoxWhisker = Options('style', boxprops=dict(color='k', linewidth=1.5),
                              whiskerprops=dict(color='k', linewidth=1.5))
@@ -239,10 +251,10 @@ options.BoxWhisker = Options('style', boxprops=dict(color='k', linewidth=1.5),
 options.Rectangles = Options('style', edgecolor='black')
 
 # Rasters
-options.Image = Options('style', cmap=dflt_cmap, interpolation='nearest')
-options.Raster = Options('style', cmap=dflt_cmap, interpolation='nearest')
-options.QuadMesh = Options('style', cmap=dflt_cmap)
-options.HeatMap = Options('style', cmap='RdYlBu_r', edgecolors='white',
+options.Image = Options('style', cmap=config.default_gridded_cmap, interpolation='nearest')
+options.Raster = Options('style', cmap=config.default_gridded_cmap, interpolation='nearest')
+options.QuadMesh = Options('style', cmap=config.default_gridded_cmap)
+options.HeatMap = Options('style', cmap=config.default_heatmap_cmap, edgecolors='white',
                           annular_edgecolors='white', annular_linewidth=0.5,
                           xmarks_edgecolor='white', xmarks_linewidth=3,
                           ymarks_edgecolor='white', ymarks_linewidth=3,
@@ -261,14 +273,21 @@ options.Slope = Options('style', color=Cycle())
 options.VSpan = Options('style', alpha=0.5, facecolor=Cycle())
 options.HSpan = Options('style', alpha=0.5, facecolor=Cycle())
 options.Spline = Options('style', edgecolor=Cycle())
+options.HLines = Options('style', color=Cycle())
+options.VLines = Options('style', color=Cycle())
+options.VSpans = Options('style', alpha=0.5, facecolor=Cycle())
+options.HSpans = Options('style', alpha=0.5, facecolor=Cycle())
+options.Labels = Options('style', color=Cycle())
 
-options.Arrow = Options('style', color='k', linewidth=2, fontsize=13)
+options.Arrow = Options('style', color='k', linewidth=2, textsize=13)
 # Paths
-options.Contours = Options('style', color=Cycle(), cmap='viridis')
+options.Contours = Options('style', color=Cycle(), cmap=dflt_cmap)
 options.Contours = Options('plot', show_legend=True)
-options.Path = Options('style', color=Cycle(), cmap='viridis')
+options.Path = Options('style', color=Cycle(), cmap=dflt_cmap)
 options.Polygons = Options('style', facecolor=Cycle(), edgecolor='black',
-                           cmap='viridis')
+                           cmap=dflt_cmap)
+options.Rectangles = Options('style', cmap=dflt_cmap)
+options.Segments = Options('style', cmap=dflt_cmap)
 options.Box = Options('style', color='black')
 options.Bounds = Options('style', color='black')
 options.Ellipse = Options('style', color='black')
@@ -280,7 +299,7 @@ options.TimeSeries = Options('style', color=Cycle())
 options.Graph = Options('style', node_edgecolors='black', node_facecolors=Cycle(),
                         edge_color='black', node_size=15)
 options.TriMesh = Options('style', node_edgecolors='black', node_facecolors='white',
-                          edge_color='black', node_size=5, edge_linewidth=1)
+                          edge_color='black', node_size=5, edge_linewidth=1, cmap=dflt_cmap)
 options.Chord = Options('style', node_edgecolors='black', node_facecolors=Cycle(),
                         edge_color='black', node_size=10, edge_linewidth=0.5)
 options.Chord = Options('plot', xaxis=None, yaxis=None)
