@@ -1,30 +1,42 @@
-# -*- coding: utf-8 -*-
 """
 Unit tests for dim transforms
 """
 import pickle
 import warnings
-
-from collections import OrderedDict
 from unittest import skipIf
 
 import numpy as np
 import pandas as pd
 import param
 
+import holoviews as hv
+
 try:
-    import dask.dataframe as dd
     import dask.array as da
-except:
+    import dask.dataframe as dd
+except ImportError:
     da, dd = None, None
 
 try:
     import xarray as xr
-except:
+except ImportError:
     xr = None
 
 xr_skip = skipIf(xr is None, "xarray not available")
 
+try:
+    import spatialpandas as spd
+except ImportError:
+    spd = None
+
+try:
+    import shapely
+except ImportError:
+    shapely = None
+
+shapelib_available = skipIf(shapely is None and spd is None,
+                            'Neither shapely nor spatialpandas are available')
+
 from holoviews.core.data import Dataset
 from holoviews.element.comparison import ComparisonTestCase
 from holoviews.util.transform import dim
@@ -63,7 +75,7 @@ class TestDimTransforms(ComparisonTestCase):
         array = np.arange(100).reshape(5, 20)
         darray = xr.DataArray(
             data=array,
-            coords=OrderedDict([('x', x), ('y', y)]),
+            coords=dict([('x', x), ('y', y)]),
             dims=['y','x']
         )
         self.dataset_xarray = Dataset(darray, vdims=['z'])
@@ -71,7 +83,7 @@ class TestDimTransforms(ComparisonTestCase):
             dask_array = da.from_array(array)
             dask_da = xr.DataArray(
                 data=dask_array,
-                coords=OrderedDict([('x', x), ('y', y)]),
+                coords=dict([('x', x), ('y', y)]),
                 dims=['y','x']
             )
             self.dataset_xarray_dask = Dataset(dask_da, vdims=['z'])
@@ -467,7 +479,7 @@ class TestDimTransforms(ComparisonTestCase):
         with warnings.catch_warnings():
             # The kwargs is {'axis': None} and is already handled by the code.
             # This context manager can be removed, when it raises an TypeError instead of warning.
-            warnings.simplefilter("ignore", "Passing additional kwargs to Rolling.mean")
+            warnings.filterwarnings("ignore", "Passing additional kwargs to Rolling.mean")
             self.assert_apply(expr, self.linear_ints.rolling(1).mean())
 
 
@@ -521,3 +533,33 @@ class TestDimTransforms(ComparisonTestCase):
         expr = (((dim('float')-2)*3)**2)
         expr2 = pickle.loads(pickle.dumps(expr))
         self.assertEqual(expr, expr2)
+
+
+@shapelib_available
+def test_dataset_transform_by_spatial_select_expr_index_not_0_based():
+    """Ensure 'spatial_select' expression works when index not zero-based.
+    Use 'spatial_select' defined by four nodes to select index 104, 105.
+    Apply expression to dataset.transform to generate new 'flag' column where True
+    for the two indexes."""
+    df = pd.DataFrame({"a": [7, 3, 0.5, 2, 1, 1], "b": [3, 4, 3, 2, 2, 1]}, index=list(range(101, 107)))
+    geometry = np.array(
+        [
+            [3.0, 1.7],
+            [0.3, 1.7],
+            [0.3, 2.7],
+            [3.0, 2.7]
+        ]
+    )
+    spatial_expr = hv.dim('a', hv.element.selection.spatial_select, hv.dim('b'), geometry=geometry)
+    dataset = hv.Dataset(df)
+    df_out = dataset.transform(flag=spatial_expr).dframe()
+    expected_series = pd.Series(
+        {
+            101: False,
+            102: False,
+            103: False,
+            104: True,
+            105: True,
+            106: False}
+        )
+    pd.testing.assert_series_equal(df_out['flag'], expected_series, check_names=False)
